/*
 * Copyright (C) The MX4J Contributors.
 * All rights reserved.
 *
 * This software is distributed under the terms of the MX4J License version 1.0.
 * See the terms of the MX4J License in the documentation provided with this software.
 */

package javax.management.loading;

/**
 * Tag interface that identifies classloader MBeans that should not be added to the
 * MBeanServer's ClassLoaderRepository when registered in the MBeanServer.
 *
 * @version $Revision: 1.3 $
 * @since JMX 1.2
 */
public interface PrivateClassLoader
{
}
